@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 use Time::HiRes;
+use Time::Duration::Parse;
 
 sub new {
     my ($class, %opts) = @_;
@@ -31,6 +32,7 @@ sub time {
 sub touch {
     $_[0]->{time} = Time::HiRes::time;
     delete $_[0]->{finish_time};
+    delete $_[0]->{expires_finish_time};
 }
 
 sub actual_stamp {
@@ -53,4 +55,31 @@ sub ttl {
     }
 }
 
+sub expires {
+    my $self = shift;
+    my $expires = $_[0];
+
+    if (@_) {
+	return $expires
+	  if $expires eq 'never' || $expires eq 'now';
+
+	$self->{expires_finish_time} = Time::HiRes::time + parse_duration( $expires );
+	return $expires;
+    }
+    else {
+	return
+	    $self->{expires_finish_time}
+	    ?
+		(
+		    $self->{expires_finish_time} > Time::HiRes::time
+		    ?
+			int( $self->{expires_finish_time} - Time::HiRes::time + 0.5 ) || 'now'
+		    :
+			'now'
+		)
+	    :
+		undef;
+    }
+}
+
 1;
